@@ -63,6 +63,10 @@
 
 #include "com.ubuntu.Upstart.h"
 
+#ifdef ENABLE_CGROUPS
+#include "cgroup.h"
+#endif /* ENABLE_CGROUPS */
+
 /* Prototypes for static functions */
 static int   control_server_connect      (DBusServer *server, DBusConnection *conn);
 static void  control_disconnected        (DBusConnection *conn);
@@ -974,6 +978,64 @@ control_notify_dbus_address (void            *data,
 	return 0;
 }
 
+/**
+ * control_notify_cgroup_manager_address:
+ * @data: not used,
+ * @message: D-Bus connection and message received,
+ * @address: D-Bus address that cgroup manager is connected to.
+ *
+ * Implements the NotifyCGroupManagerAddress method of the
+ * com.ubuntu.Upstart interface.
+ *
+ * Called to allow the cgroup manager to be contacted,
+ * thus enabling the cgroup stanza.
+ *
+ * Returns: zero on success, negative value on raised error.
+ **/
+int
+control_notify_cgroup_manager_address (void            *data,
+				       NihDBusMessage  *message,
+				       const char      *address)
+{
+	nih_assert (message);
+	nih_assert (address);
+
+	if (! control_check_permission (message)) {
+		nih_dbus_error_raise_printf (
+			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
+			_("You do not have permission to notify cgroup manager address"));
+		return -1;
+	}
+
+#ifdef ENABLE_CGROUPS
+	if (! cgroup_support_enabled ())
+		return 0;
+
+	/* Already called */
+	if (cgroup_manager_available ())
+		return 0;
+
+	if (! cgroup_manager_set_address (address)) {
+		nih_dbus_error_raise_printf (DBUS_ERROR_NO_MEMORY,
+				_("Out of Memory"));
+		return -1;
+	}
+
+	nih_debug ("set cgroup manager address");
+
+	if (! job_class_induct_jobs ()) {
+		nih_dbus_error_raise_printf (DBUS_ERROR_NO_MEMORY,
+				_("Out of Memory"));
+		return -1;
+	}
+
+#else
+	nih_debug ("cgroup support not available");
+#endif /* ENABLE_CGROUPS */
+
+	return 0;
+}
+
 /**
  * control_bus_flush:
  *
@@ -1252,6 +1314,121 @@ control_notify_restarted (void)
 	}
 }
 
+/**
+ * control_set_env_list:
+ *
+ * @data: not used,
+ * @message: D-Bus connection and message received,
+ * @job_details: name and instance of job to apply operation to,
+ * @vars: array of name[/value] pairs of environment variables to set,
+ * @replace: TRUE if @name should be overwritten if already set, else
+ *  FALSE.
+ *
+ * Implements the SetEnvList method of the com.ubuntu.Upstart
+ * interface.
+ *
+ * Called to request Upstart store one or more name/value pairs.
+ *
+ * If @job_details is empty, change will be applied to all job
+ * environments, else only apply changes to specific job environment
+ * encoded within @job_details.
+ *
+ * Returns: zero on success, negative value on raised error.
+ **/
+int
+control_set_env_list (void            *data,
+		      NihDBusMessage  *message,
+		      char * const    *job_details,
+		      char * const    *vars,
+		      int              replace)
+{
+	Session         *session;
+	Job             *job = NULL;
+	char            *job_name = NULL;
+	char            *instance = NULL;
+	char * const    *var;
+
+	nih_assert (message);
+	nih_assert (job_details);
+	nih_assert (vars);
+
+	if (! control_check_permission (message)) {
+		nih_dbus_error_raise_printf (
+			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
+			_("You do not have permission to modify job environment"));
+		return -1;
+	}
+
+	if (getpid () == 1) {
+		nih_dbus_error_raise_printf (
+			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
+			_("Not permissible to modify PID 1 job environment"));
+		return -1;
+	}
+
+	if (job_details[0]) {
+		job_name = job_details[0];
+
+		/* this can be a null value */
+		instance = job_details[1];
+	}
+
+	/* Verify that job name is valid */
+	if (job_name && ! strlen (job_name)) {
+		nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
+					     _("Job may not be empty string"));
+		return -1;
+	}
+
+	/* Get the relevant session */
+	session = session_from_dbus (NULL, message);
+
+	/* Chroot sessions must not be able to influence
+	 * the outside system.
+	 */
+	if (session && session->chroot) {
+		nih_warn (_("Ignoring set env request from chroot session"));
+		return 0;
+	}
+
+	/* Lookup the job */
+	control_get_job (session, job, job_name, instance);
+
+	for (var = vars; var && *var; var++) {
+		nih_local char *envvar = NULL;
+
+		if (! *var) {
+			nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
+					_("Variable may not be empty string"));
+			return -1;
+		}
+
+		/* If variable does not contain a delimiter, add one to ensure
+		 * it gets entered into the job environment table. Without the
+		 * delimiter, the variable will be silently ignored unless it's
+		 * already set in inits environment. But in that case there is
+		 * no point in setting such a variable to its already existing
+		 * value.
+		 */
+		if (! strchr (*var, '=')) {
+			envvar = NIH_MUST (nih_sprintf (NULL, "%s=", *var));
+		} else {
+			envvar = NIH_MUST (nih_strdup (NULL, *var));
+		}
+
+		if (job) {
+			/* Modify job-specific environment */
+			nih_assert (job->env);
+
+			NIH_MUST (environ_add (&job->env, job, NULL, replace, envvar));
+		} else if (job_class_environment_set (envvar, replace) < 0) {
+			nih_return_no_memory_error (-1);
+		}
+	}
+
+	return 0;
+}
+
 /**
  * control_set_env:
  *
@@ -1279,15 +1456,53 @@ control_set_env (void            *data,
 		 char * const    *job_details,
 		 const char      *var,
 		 int              replace)
+{
+	nih_local char **vars = NULL;
+	
+	if (! var) {
+		nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
+					_("Variable may not be empty string"));
+		return -1;
+	}
+
+	vars = NIH_MUST (nih_str_array_new (NULL));
+
+	NIH_MUST (nih_str_array_add (&vars, NULL, NULL, var));
+
+	return control_set_env_list (data, message, job_details, vars, replace);
+}
+
+/**
+ * control_unset_env_list:
+ *
+ * @data: not used,
+ * @message: D-Bus connection and message received,
+ * @job_details: name and instance of job to apply operation to,
+ * @names: array of variables to clear from the job environment array.
+ *
+ * Implements the UnsetEnvList method of the com.ubuntu.Upstart
+ * interface.
+ *
+ * Called to request Upstart remove one or more variables from the job
+ * environment array.
+ *
+ * Returns: zero on success, negative value on raised error.
+ **/
+int
+control_unset_env_list (void            *data,
+			NihDBusMessage  *message,
+			char * const    *job_details,
+			char * const    *names)
 {
 	Session         *session;
 	Job             *job = NULL;
 	char            *job_name = NULL;
 	char            *instance = NULL;
-	nih_local char  *envvar = NULL;
+	char * const    *name;
 
 	nih_assert (message);
 	nih_assert (job_details);
+	nih_assert (names);
 
 	if (! control_check_permission (message)) {
 		nih_dbus_error_raise_printf (
@@ -1296,22 +1511,18 @@ control_set_env (void            *data,
 		return -1;
 	}
 
-	if (job_details[0]) {
-		job_name = job_details[0];
-
-		/* this can be a null value */
-		instance = job_details[1];
-	} else if (getpid () == 1) {
+	if (getpid () == 1) {
 		nih_dbus_error_raise_printf (
 			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
 			_("Not permissible to modify PID 1 job environment"));
 		return -1;
 	}
 
-	if (! var || ! *var) {
-		nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
-				_("Variable may not be empty string"));
-		return -1;
+	if (job_details[0]) {
+		job_name = job_details[0];
+
+		/* this can be a null value */
+		instance = job_details[1];
 	}
 
 	/* Verify that job name is valid */
@@ -1328,38 +1539,38 @@ control_set_env (void            *data,
 	 * the outside system.
 	 */
 	if (session && session->chroot) {
-		nih_warn (_("Ignoring set env request from chroot session"));
+		nih_warn (_("Ignoring unset env request from chroot session"));
 		return 0;
 	}
 
 	/* Lookup the job */
 	control_get_job (session, job, job_name, instance);
 
-	/* If variable does not contain a delimiter, add one to ensure
-	 * it gets entered into the job environment table. Without the
-	 * delimiter, the variable will be silently ignored unless it's
-	 * already set in inits environment. But in that case there is
-	 * no point in setting such a variable to its already existing
-	 * value.
-	 */
-	if (! strchr (var, '='))
-		envvar = NIH_MUST (nih_sprintf (NULL, "%s=", var));
-	else
-		envvar = NIH_MUST (nih_strdup (NULL, var));
+	for (name = names; name && *name; name++) {
+		if (! *name) {
+			nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
+					_("Variable may not be empty string"));
+			return -1;
+		}
 
-	if (job) {
-		/* Modify job-specific environment */
+		if (job) {
+			/* Modify job-specific environment */
+			nih_assert (job->env);
 
-		nih_assert (job->env);
-
-		NIH_MUST (environ_add (&job->env, job, NULL, replace, envvar));
-		return 0;
+			if (! environ_remove (&job->env, job, NULL, *name))
+				return -1;
+		} else if (job_class_environment_unset (*name) < 0) {
+			goto error;
+		}
 	}
 
-	if (job_class_environment_set (envvar, replace) < 0)
-		nih_return_no_memory_error (-1);
-
 	return 0;
+
+error:
+	nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
+			"%s: %s",
+			_("No such variable"), *name);
+	return -1;
 }
 
 /**
@@ -1384,81 +1595,19 @@ control_unset_env (void            *data,
 		   char * const    *job_details,
 		   const char      *name)
 {
-	Session         *session;
-	Job             *job = NULL;
-	char            *job_name = NULL;
-	char            *instance = NULL;
+	nih_local char **names = NULL;
 
-	nih_assert (message);
-	nih_assert (job_details);
-
-	if (! control_check_permission (message)) {
-		nih_dbus_error_raise_printf (
-			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
-			_("You do not have permission to modify job environment"));
-		return -1;
-	}
-
-	if (! name || ! *name) {
+	if (! name) {
 		nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
-				_("Variable may not be empty string"));
+					_("Variable may not be empty string"));
 		return -1;
 	}
+	
+	names = NIH_MUST (nih_str_array_new (NULL));
 
-	if (job_details[0]) {
-		job_name = job_details[0];
+	NIH_MUST (nih_str_array_add (&names, NULL, NULL, name));
 
-		/* this can be a null value */
-		instance = job_details[1];
-	} else if (getpid () == 1) {
-		nih_dbus_error_raise_printf (
-			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
-			_("Not permissible to modify PID 1 job environment"));
-		return -1;
-	}
-
-	/* Verify that job name is valid */
-	if (job_name && ! strlen (job_name)) {
-		nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
-					     _("Job may not be empty string"));
-		return -1;
-	}
-
-	/* Get the relevant session */
-	session = session_from_dbus (NULL, message);
-
-	/* Chroot sessions must not be able to influence
-	 * the outside system.
-	 */
-	if (session && session->chroot) {
-		nih_warn (_("Ignoring unset env request from chroot session"));
-		return 0;
-	}
-
-	/* Lookup the job */
-	control_get_job (session, job, job_name, instance);
-
-	if (job) {
-		/* Modify job-specific environment */
-
-		nih_assert (job->env);
-
-		if (! environ_remove (&job->env, job, NULL, name))
-			return -1;
-
-		return 0;
-	}
-
-	if (job_class_environment_unset (name) < 0)
-		goto error;
-
-	return 0;
-
-error:
-	nih_dbus_error_raise_printf (DBUS_ERROR_INVALID_ARGS,
-			"%s: %s",
-			_("No such variable"), name);
-	return -1;
+	return control_unset_env_list (data, message, job_details, names);
 }
 
 /**
@@ -1672,16 +1821,18 @@ control_reset_env (void           *data,
 		return -1;
 	}
 
+	if (getpid () == 1) {
+		nih_dbus_error_raise_printf (
+			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
+			_("Not permissible to modify PID 1 job environment"));
+		return -1;
+	}
+
 	if (job_details[0]) {
 		job_name = job_details[0];
 
 		/* this can be a null value */
 		instance = job_details[1];
-	} else if (getpid () == 1) {
-		nih_dbus_error_raise_printf (
-			DBUS_INTERFACE_UPSTART ".Error.PermissionDenied",
-			_("Not permissible to modify PID 1 job environment"));
-		return -1;
 	}
 
 	/* Verify that job name is valid */
