@@ -48,6 +48,9 @@
 
 #include "com.ubuntu.Upstart.h"
 
+#ifdef ENABLE_CGROUPS
+#include "cgroup.h"
+#endif /* ENABLE_CGROUPS */
 
 /* Prototypes for static functions */
 static void event_pending              (Event *event);
@@ -302,6 +305,10 @@ event_pending_handle_jobs (Event *event)
 {
 	int  empty = TRUE;
 
+#ifdef ENABLE_CGROUPS
+	int  warn = FALSE;
+#endif /* ENABLE_CGROUPS */
+
 	nih_assert (event != NULL);
 
 	job_class_init ();
@@ -365,77 +372,59 @@ event_pending_handle_jobs (Event *event)
 
 		}
 
+		/* If the job has specified a cgroup stanza, do not
+		 * start it until the cgroup manager is available. Also,
+		 * block any events that the job requires such that when
+		 * the cgroup manager is available, the job may be
+		 * started.
+		 */
+
+#ifdef ENABLE_CGROUPS
+		if (class->start_on && job_class_cgroups (class)) {
+			if (cgroup_manager_available ()) {
+
+				if (class->cgmanager_wait) {
+					/* Unref the events that were ref'ed
+					 * whilst waiting for the cgroup manager
+					 * to become available.
+					 */
+					event_operator_reset (class->start_on);
+					class->cgmanager_wait = FALSE;
+				}
+			} else {
+				warn = TRUE;
+
+				/* Reference the event to stop it being destroyed since it will
+				 * be required by the job once the cgroup manager eventually
+				 * becomes available.
+				 */
+				if (! class->cgmanager_wait) {
+					if (event_operator_handle (class->start_on, event, NULL))
+						class->cgmanager_wait = TRUE;
+				}
+
+				continue;
+			}
+		}
+#endif /* ENABLE_CGROUPS */
+
 		/* Now we match the start events for the class to see
 		 * whether we need a new instance.
 		 */
 		if (class->start_on
 		    && event_operator_handle (class->start_on, event, NULL)
 		    && class->start_on->value) {
-			nih_local char **env = NULL;
-			nih_local char  *name = NULL;
-			size_t           len;
-			Job             *job;
 
-			/* Construct the environment for the new instance
-			 * from the class and the start events.
-			 */
-			env = NIH_MUST (job_class_environment (
-					  NULL, class, &len));
-			NIH_MUST (event_operator_environment (class->start_on,
-							      &env, NULL, &len,
-							      "UPSTART_EVENTS"));
-
-			/* Expand the instance name against the environment */
-			name = NIH_SHOULD (environ_expand (NULL,
-							   class->instance,
-							   env));
-			if (! name) {
-				NihError *err;
-
-				err = nih_error_get ();
-				nih_warn (_("Failed to obtain %s instance: %s"),
-					  class->name, err->message);
-				nih_free (err);
-
-				event_operator_reset (class->start_on);
-				continue;
-			}
-
-			/* Locate the current instance or create a new one */
-			job = (Job *)nih_hash_lookup (class->instances, name);
-			if (! job)
-				job = NIH_MUST (job_new (class, name));
-
-			nih_debug ("New instance %s", job_name (job));
-
-			/* Start the job with the environment we want */
-			if (job->goal != JOB_START) {
-				if (job->start_env)
-					nih_unref (job->start_env, job);
-
-				job->start_env = env;
-				nih_ref (job->start_env, job);
-
-				nih_discard (env);
-				env = NULL;
-
-				job_finished (job, FALSE);
-
-				NIH_MUST (event_operator_fds (class->start_on, job,
-							      &job->fds, &job->num_fds,
-							      &job->start_env, &len,
-							      "UPSTART_FDS"));
-
-				event_operator_events (job->class->start_on,
-						       job, &job->blocking);
-
-				job_change_goal (job, JOB_START);
-			}
-
-			event_operator_reset (class->start_on);
+			if (! job_class_induct_job (class))
+				return;
 		}
 	}
 
+#ifdef ENABLE_CGROUPS
+	if (warn)
+		nih_debug ("Cannot start some jobs until cgroup manager available");
+#endif /* ENABLE_CGROUPS */
+
 	if (! quiesce_in_progress ())
 		return;
 
@@ -450,7 +439,7 @@ event_pending_handle_jobs (Event *event)
 
 		if (! empty)
 			break;
-	}
+	}	
 
 	/* If no instances remain, force quiesce to finish */
 	if (empty)
@@ -672,7 +661,7 @@ event_deserialise (json_object *json)
 	if (! state_get_json_string_var_strict (json, "name", NULL, name))
 		goto error;
 
-	if (json_object_object_get (json, "env")) {
+	if (json_object_object_get_ex (json, "env", NULL)) {
 		if (! state_get_json_var_full (json, "env", array, json_env))
 			goto error;
 
@@ -706,7 +695,7 @@ event_deserialise (json_object *json)
 	 * possible to manually reconstruct the state of the
 	 * EventOperators post-re-exec.
 	 */
-	if (json_object_object_get (json, "blockers")) {
+	if (json_object_object_get_ex (json, "blockers", NULL)) {
 		if (! state_get_json_int_var_to_obj (json, event, blockers))
 			goto error;
 	}
@@ -737,9 +726,8 @@ event_deserialise_all (json_object *json)
 	event_init ();
 
 	nih_assert (NIH_LIST_EMPTY (events));
-	json_events = json_object_object_get (json, "events");
 
-	if (! json_events)
+	if (! json_object_object_get_ex (json, "events", &json_events))
 		goto error;
 
 	if (! state_check_json_type (json_events, array))
