@@ -1,6 +1,6 @@
 /* upstart
  *
- * Copyright © 2009-2011 Canonical Ltd.
+ * Copyright  2009-2011 Canonical Ltd.
  * Author: Scott James Remnant <scott@netsplit.com>.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -85,10 +85,12 @@ static void hup_handler     (void *data, NihSignal *signal);
 static void usr1_handler    (void *data, NihSignal *signal);
 #endif /* DEBUG */
 
-static void handle_confdir      (void);
-static void handle_logdir       (void);
-static int  console_type_setter (NihOption *option, const char *arg);
-static int  conf_dir_setter     (NihOption *option, const char *arg);
+static void handle_confdir          (void);
+static void handle_logdir           (void);
+static int  console_type_setter     (NihOption *option, const char *arg);
+static int  conf_dir_setter         (NihOption *option, const char *arg);
+static int  prepend_conf_dir_setter (NihOption *option, const char *arg);
+static int  append_conf_dir_setter  (NihOption *option, const char *arg);
 
 
 /**
@@ -106,6 +108,22 @@ static int state_fd = -1;
  **/
 static char **conf_dirs = NULL;
 
+/**
+ * prepend_conf_dirs:
+ *
+ * Array of full paths to job configuration file directories that will
+ * be added to before the other values in conf_dirs.
+ **/
+static char **prepend_conf_dirs = NULL;
+
+/**
+ * append_conf_dirs:
+ *
+ * Array of full paths to job configuration file directories that will
+ * be added to conf_dirs.
+ **/
+static char **append_conf_dirs = NULL;
+
 /**
  * initial_event:
  *
@@ -130,7 +148,7 @@ static int disable_dbus = FALSE;
 
 extern int          no_inherit_env;
 extern int          user_mode;
-extern int          disable_sessions;
+extern int          chroot_sessions;
 extern int          disable_job_logging;
 extern int          use_session_bus;
 extern int          default_console;
@@ -140,36 +158,51 @@ extern DBusBusType  dbus_bus_type;
 extern mode_t       initial_umask;
 extern int          debug_stanza_enabled;
 
+#ifdef ENABLE_CGROUPS
+extern int          disable_cgroups;
+#endif /* ENABLE_CGROUPS */
+
 /**
  * options:
  *
  * Command-line options we accept.
  **/
 static NihOption options[] = {
+	{ 0, "append-confdir", N_("specify additional directory to load configuration files from"),
+		NULL, "DIR", NULL, append_conf_dir_setter },
+
+	{ 0, "chroot-sessions", N_("enable chroot sessions"),
+		NULL, NULL, &chroot_sessions, NULL },
+
 	{ 0, "confdir", N_("specify alternative directory to load configuration files from"),
 		NULL, "DIR", NULL, conf_dir_setter },
 
 	{ 0, "default-console", N_("default value for console stanza"),
 		NULL, "VALUE", NULL, console_type_setter },
 
+	{ 0, "logdir", N_("specify alternative directory to store job output logs in"),
+		NULL, "DIR", &log_dir, NULL },
+
+#ifdef ENABLE_CGROUPS
+	{ 0, "no-cgroups", N_("do not support cgroups"),
+		NULL, NULL, &disable_cgroups, NULL },
+#endif /* ENABLE_CGROUPS */
+
 	{ 0, "no-dbus", N_("do not connect to a D-Bus bus"),
 		NULL, NULL, &disable_dbus, NULL },
 
 	{ 0, "no-inherit-env", N_("jobs will not inherit environment of init"),
 		NULL, NULL, &no_inherit_env , NULL },
 
-	{ 0, "logdir", N_("specify alternative directory to store job output logs in"),
-		NULL, "DIR", &log_dir, NULL },
-
 	{ 0, "no-log", N_("disable job logging"),
 		NULL, NULL, &disable_job_logging, NULL },
 
-	{ 0, "no-sessions", N_("disable chroot sessions"),
-		NULL, NULL, &disable_sessions, NULL },
-
 	{ 0, "no-startup-event", N_("do not emit any startup event (for testing)"),
 		NULL, NULL, &disable_startup_event, NULL },
 
+	{ 0, "prepend-confdir", N_("specify additional initial directory to load configuration files from"),
+		NULL, "DIR", NULL, prepend_conf_dir_setter },
+
 	/* Must be specified for both stateful and stateless re-exec */
 	{ 0, "restart", N_("flag a re-exec has occurred"),
 		NULL, NULL, &restart, NULL },
@@ -205,6 +238,8 @@ main (int   argc,
 	int    ret;
 
 	conf_dirs = NIH_MUST (nih_str_array_new (NULL));
+	append_conf_dirs = NIH_MUST (nih_str_array_new (NULL));
+	prepend_conf_dirs = NIH_MUST (nih_str_array_new (NULL));
 
 	args_copy = NIH_MUST (nih_str_array_copy (NULL, NULL, argv));
 
@@ -566,43 +601,58 @@ main (int   argc,
 		}
 	}
 
-	/* Read configuration */
-	if (! user_mode) {
-		char   *conf_dir;
-		int     len = 0;
+	/* Only honour command-line options affecting configuration
+	 * directories if not restarting, or if performing a stateless
+	 * re-exec.
+	 */
+	if (! restart || (restart && state_fd == -1)) {
+		/* Read configuration */
+		if (prepend_conf_dirs[0]) {
+			for (char **d = prepend_conf_dirs; d && *d; d++) {
+				nih_debug ("Prepending configuration directory %s", *d);
+				NIH_MUST (conf_source_new (NULL, *d, CONF_JOB_DIR));
+			}
+		}
 
-		nih_assert (conf_dirs[0]);
+		if (! user_mode) {
+			nih_assert (conf_dirs[0]);
 
-		/* Count entries */
-		for (char **d = conf_dirs; d && *d; d++, len++)
-			;
+			NIH_MUST (conf_source_new (NULL, CONFFILE, CONF_FILE));
 
-		nih_assert (len);
+			for (char **d = conf_dirs; d && *d; d++) {
+				nih_debug ("Using configuration directory %s", *d);
+				NIH_MUST (conf_source_new (NULL, *d, CONF_JOB_DIR));
+			}
+		} else {
+			nih_local char **dirs = NULL;
 
-		/* Use last value specified */
-		conf_dir = conf_dirs[len-1];
+			dirs = NIH_MUST (get_user_upstart_dirs ());
 
-		NIH_MUST (conf_source_new (NULL, CONFFILE, CONF_FILE));
+			for (char **d = conf_dirs[0] ? conf_dirs : dirs; d && *d; d++) {
+				nih_debug ("Using configuration directory %s", *d);
+				NIH_MUST (conf_source_new (NULL, *d, CONF_JOB_DIR));
+			}
+		}
 
-		nih_debug ("Using configuration directory %s", conf_dir);
-		NIH_MUST (conf_source_new (NULL, conf_dir, CONF_JOB_DIR));
-	} else {
-		nih_local char **dirs = NULL;
-
-		dirs = NIH_MUST (get_user_upstart_dirs ());
-
-		for (char **d = conf_dirs[0] ? conf_dirs : dirs; d && *d; d++) {
-			nih_debug ("Using configuration directory %s", *d);
-			NIH_MUST (conf_source_new (NULL, *d, CONF_JOB_DIR));
+		if (append_conf_dirs[0]) {
+			for (char **d = append_conf_dirs; d && *d; d++) {
+				nih_debug ("Adding configuration directory %s", *d);
+				NIH_MUST (conf_source_new (NULL, *d, CONF_JOB_DIR));
+			}
 		}
 	}
 
 	nih_free (conf_dirs);
+	nih_free (prepend_conf_dirs);
+	nih_free (append_conf_dirs);
 
 	job_class_environment_init ();
 
 	conf_reload ();
 
+	/* We must have atleast one source of configuration */
+	nih_assert (! NIH_LIST_EMPTY (conf_sources));
+
 	/* Create a listening server for private connections. */
 	if (use_session_bus == FALSE) {
 		while (control_server_open () < 0) {
@@ -696,8 +746,8 @@ main (int   argc,
 		control_notify_restarted();
 	}
 
-	if (disable_sessions)
-		nih_debug ("Chroot Sessions disabled");
+	if (chroot_sessions)
+		nih_debug ("Chroot Sessions enabled");
 
 	/* Set us as the child subreaper.
 	 * This ensures that even when init doesn't run as PID 1, it'll always be
@@ -1105,3 +1155,37 @@ conf_dir_setter (NihOption *option, const char *arg)
 
 	return 0;
 }
+
+/**  
+ * NihOption setter function to handle selection of configuration file
+ * directories.
+ *
+ * Returns: 0 on success, -1 on invalid console type.
+ **/
+static int
+prepend_conf_dir_setter (NihOption *option, const char *arg)
+{
+	nih_assert (prepend_conf_dirs);
+	nih_assert (option);
+
+	NIH_MUST (nih_str_array_add (&prepend_conf_dirs, NULL, NULL, arg));
+
+	return 0;
+}
+
+/**  
+ * NihOption setter function to handle selection of configuration file
+ * directories.
+ *
+ * Returns: 0 on success, -1 on invalid console type.
+ **/
+static int
+append_conf_dir_setter (NihOption *option, const char *arg)
+{
+	nih_assert (append_conf_dirs);
+	nih_assert (option);
+
+	NIH_MUST (nih_str_array_add (&append_conf_dirs, NULL, NULL, arg));
+
+	return 0;
+}
