@@ -33,14 +33,11 @@
  * Session:
  * @entry: list header,
  * @chroot: path all jobs are chrooted to,
- * @user: uid all jobs are switched to,
- * @conf_path: configuration path (either full path to chroot root, or
- * full path to users job directory (which may itself be prepended
- * with a chroot path)).
+ * @conf_path: configuration path (full path to chroot root).
  *
  * This structure is used to identify collections of jobs
- * that share either a common @chroot and/or common @user. Note that
- * @conf_path is unique across all sessions.
+ * that share a common @chroot (*). Note that @conf_path is
+ * unique across all sessions.
  *
  * Summary of Session values for different environments:
  *
@@ -50,27 +47,28 @@
  * | user | PID  | chroot | uid | Object contents                   |
  * +------+------+--------+-----+-----------------------------------+
  * | 0    | >0   | no     | 0   | NULL (*1)                         |
- * | >0   | "0"  | no     | >0  | uid + conf_path set to "~/.init". |
  * | 0    | >0   | yes    | 0   | chroot + conf_path set            |
- * | >0   | ??   | yes    | >0  | XXX: fails (*2)                   |
+ * | >0   | ??   | yes    | >0  | Not permitted (*2)                |
  * +------+------+--------+-----+-----------------------------------+
  *
  * Notes:
  *
+ * (*) - this structure used to also store user session details (hence
+ * the name), but the functionality was removed with the advent of
+ * a true user mode.
+ *
  * (*1) - The "NULL session" represents the "traditional" environment
  * before sessions were introduced (namely a non-chroot environment
  * where all job and event operations were handled by uid 0 (root)).
  *
- * (*2) - error is:
+ * (*2) - User lookup is not reliable since the user to query exists
+ * within the chroot, but the only possible lookup is outside the
+ * chroot.
  *
- *   initctl: Unable to connect to system bus: Failed to connect to socket
- *   /var/run/dbus/system_bus_socket: No such file or directory
- * 
  **/
 typedef struct session {
 	NihList entry;
 	char *  chroot;
-	uid_t   user;
 	char *  conf_path;
 } Session;
 
@@ -80,14 +78,15 @@ NIH_BEGIN_EXTERN
 extern NihList *sessions;
 
 void           session_init        (void);
+void           session_destroy     (void);
 
-Session      * session_new         (const void *parent, const char *chroot, uid_t user)
-	__attribute__ ((malloc, warn_unused_result));
+Session      * session_new         (const void *parent, const char *chroot)
+	__attribute__ ((warn_unused_result));
 
 Session      * session_from_dbus   (const void *parent, NihDBusMessage *message);
 
 json_object  * session_serialise_all   (void)
-	__attribute__ ((malloc, warn_unused_result));
+	__attribute__ ((warn_unused_result));
 
 int            session_deserialise_all (json_object *json)
 	__attribute__ ((warn_unused_result));
