@@ -1144,7 +1144,7 @@ int  state_to_string (char **json_string, size_t *len)
 int    state_from_string (const char *state)
 	__attribute__ ((warn_unused_result));
 
-int    state_toggle_cloexec (int fd, int set);
+int    state_modify_cloexec (int fd, int set);
 
 json_object *
 state_serialise_str_array (char ** const array)
