@@ -618,8 +618,10 @@ test_add (void)
 void
 test_remove (void)
 {
-	char   **env = NULL, **ret;
-	size_t   len = 0;
+	char    **env = NULL;
+	char     *removed = NULL;
+	char    **ret = NULL;
+	size_t    len = 0;
 
 	TEST_FUNCTION ("environ_remove");
 
@@ -688,6 +690,9 @@ test_remove (void)
 			TEST_ALLOC_PARENT (env[0], env);
 			TEST_ALLOC_SIZE (env[0], 8);
 			TEST_EQ_STR (env[0], "FOO=BAR");
+			removed = env[0];
+			TEST_FREE_TAG (removed);
+
 			TEST_EQ_P (env[1], NULL);
 		}
 
@@ -695,17 +700,6 @@ test_remove (void)
 
 		if (test_alloc_failed) {
 			TEST_EQ_P (ret, NULL);
-
-			TEST_EQ (len, 1);
-
-			TEST_ALLOC_PARENT (env[0], env);
-			TEST_ALLOC_SIZE (env[0], 8);
-
-			TEST_NE_P (env[0], NULL);
-			TEST_EQ_STR (env[0], "FOO=BAR");
-
-			TEST_EQ_P (env[1], NULL);
-
 			nih_free (env);
 			continue;
 		}
@@ -713,6 +707,7 @@ test_remove (void)
 		TEST_NE_P (ret, NULL);
 		TEST_EQ (len, 0);
 		TEST_EQ_P (env[0], NULL);
+		TEST_FREE (removed);
 
 		nih_free (env);
 	}
@@ -730,6 +725,9 @@ test_remove (void)
 			TEST_ALLOC_PARENT (env[0], env);
 			TEST_ALLOC_SIZE (env[0], 8);
 			TEST_EQ_STR (env[0], "FOO=BAR");
+			removed = env[0];
+			TEST_FREE_TAG (removed);
+
 			TEST_EQ_P (env[1], NULL);
 		}
 
@@ -737,17 +735,6 @@ test_remove (void)
 
 		if (test_alloc_failed) {
 			TEST_EQ_P (ret, NULL);
-
-			TEST_EQ (len, 1);
-
-			TEST_ALLOC_PARENT (env[0], env);
-			TEST_ALLOC_SIZE (env[0], 8);
-
-			TEST_NE_P (env[0], NULL);
-			TEST_EQ_STR (env[0], "FOO=BAR");
-
-			TEST_EQ_P (env[1], NULL);
-
 			nih_free (env);
 			continue;
 		}
@@ -755,6 +742,7 @@ test_remove (void)
 		TEST_NE_P (ret, NULL);
 		TEST_EQ (len, 0);
 		TEST_EQ_P (env[0], NULL);
+		TEST_FREE (removed);
 
 		nih_free (env);
 	}
@@ -781,6 +769,8 @@ test_remove (void)
 			TEST_ALLOC_PARENT (env[0], env);
 			TEST_ALLOC_SIZE (env[0], 8);
 			TEST_EQ_STR (env[0], "FOO=BAR");
+			removed = env[0];
+			TEST_FREE_TAG (removed);
 
 			TEST_ALLOC_PARENT (env[1], env);
 			TEST_ALLOC_SIZE (env[1], 8);
@@ -794,18 +784,6 @@ test_remove (void)
 
 		if (test_alloc_failed) {
 			TEST_EQ_P (ret, NULL);
-
-			TEST_EQ (len, 2);
-			TEST_ALLOC_PARENT (env[0], env);
-			TEST_ALLOC_SIZE (env[0], 8);
-			TEST_EQ_STR (env[0], "FOO=BAR");
-
-			TEST_ALLOC_PARENT (env[1], env);
-			TEST_ALLOC_SIZE (env[1], 8);
-			TEST_EQ_STR (env[1], "BAZ=QUX");
-
-			TEST_EQ_P (env[2], NULL);
-
 			nih_free (env);
 			continue;
 		}
@@ -816,6 +794,7 @@ test_remove (void)
 		TEST_ALLOC_PARENT (env[0], env);
 		TEST_ALLOC_SIZE (env[0], 8);
 		TEST_EQ_STR (env[0], "BAZ=QUX");
+		TEST_FREE (removed);
 
 		TEST_EQ_P (env[1], NULL);
 
@@ -852,6 +831,8 @@ test_remove (void)
 			TEST_ALLOC_PARENT (env[0], env);
 			TEST_ALLOC_SIZE (env[0], 8);
 			TEST_EQ_STR (env[0], "UPSTART_TEST_VARIABLE=foo");
+			removed = env[0];
+			TEST_FREE_TAG (removed);
 
 			TEST_ALLOC_PARENT (env[1], env);
 			TEST_ALLOC_SIZE (env[1], 8);
@@ -865,18 +846,6 @@ test_remove (void)
 
 		if (test_alloc_failed) {
 			TEST_EQ_P (ret, NULL);
-
-			TEST_EQ (len, 2);
-			TEST_ALLOC_PARENT (env[0], env);
-			TEST_ALLOC_SIZE (env[0], 8);
-			TEST_EQ_STR (env[0], "UPSTART_TEST_VARIABLE=foo");
-
-			TEST_ALLOC_PARENT (env[1], env);
-			TEST_ALLOC_SIZE (env[1], 8);
-			TEST_EQ_STR (env[1], "BAZ=QUX");
-
-			TEST_EQ_P (env[2], NULL);
-
 			nih_free (env);
 			continue;
 		}
@@ -887,6 +856,7 @@ test_remove (void)
 		TEST_ALLOC_PARENT (env[0], env);
 		TEST_ALLOC_SIZE (env[0], 8);
 		TEST_EQ_STR (env[0], "BAZ=QUX");
+		TEST_FREE (removed);
 
 		TEST_EQ_P (env[1], NULL);
 
@@ -921,6 +891,8 @@ test_remove (void)
 			TEST_ALLOC_PARENT (env[1], env);
 			TEST_ALLOC_SIZE (env[1], 8);
 			TEST_EQ_STR (env[1], "BAZ=QUX");
+			removed = env[1];
+			TEST_FREE_TAG (removed);
 
 			TEST_EQ_P (env[2], NULL);
 		}
@@ -930,18 +902,6 @@ test_remove (void)
 
 		if (test_alloc_failed) {
 			TEST_EQ_P (ret, NULL);
-
-			TEST_EQ (len, 2);
-			TEST_ALLOC_PARENT (env[0], env);
-			TEST_ALLOC_SIZE (env[0], 8);
-			TEST_EQ_STR (env[0], "FOO=BAR");
-
-			TEST_ALLOC_PARENT (env[1], env);
-			TEST_ALLOC_SIZE (env[1], 8);
-			TEST_EQ_STR (env[1], "BAZ=QUX");
-
-			TEST_EQ_P (env[2], NULL);
-
 			nih_free (env);
 			continue;
 		}
@@ -954,6 +914,7 @@ test_remove (void)
 		TEST_EQ_STR (env[0], "FOO=BAR");
 
 		TEST_EQ_P (env[1], NULL);
+		TEST_FREE (removed);
 
 		nih_free (env);
 	}
@@ -991,6 +952,9 @@ test_remove (void)
 			/* Should have been expanded */
 			TEST_EQ_STR (env[1], "UPSTART_TEST_VARIABLE=foo");
 
+			removed = env[1];
+			TEST_FREE_TAG (removed);
+
 			TEST_EQ_P (env[2], NULL);
 		}
 
@@ -999,18 +963,6 @@ test_remove (void)
 
 		if (test_alloc_failed) {
 			TEST_EQ_P (ret, NULL);
-
-			TEST_EQ (len, 2);
-			TEST_ALLOC_PARENT (env[0], env);
-			TEST_ALLOC_SIZE (env[0], 8);
-			TEST_EQ_STR (env[0], "FOO=BAR");
-
-			TEST_ALLOC_PARENT (env[1], env);
-			TEST_ALLOC_SIZE (env[1], 8);
-			TEST_EQ_STR (env[1], "UPSTART_TEST_VARIABLE=foo");
-
-			TEST_EQ_P (env[2], NULL);
-
 			nih_free (env);
 			continue;
 		}
@@ -1023,6 +975,7 @@ test_remove (void)
 		TEST_EQ_STR (env[0], "FOO=BAR");
 
 		TEST_EQ_P (env[1], NULL);
+		TEST_FREE (removed);
 
 		nih_free (env);
 	}
