@@ -119,26 +119,28 @@ static int    ignored_events_setter (NihOption *option, const char *arg);
 #endif
 
 /* Prototypes for option and command functions */
-int start_action                  (NihCommand *command, char * const *args);
-int stop_action                   (NihCommand *command, char * const *args);
-int restart_action                (NihCommand *command, char * const *args);
-int reload_action                 (NihCommand *command, char * const *args);
-int status_action                 (NihCommand *command, char * const *args);
-int list_action                   (NihCommand *command, char * const *args);
-int emit_action                   (NihCommand *command, char * const *args);
-int reload_configuration_action   (NihCommand *command, char * const *args);
-int version_action                (NihCommand *command, char * const *args);
-int log_priority_action           (NihCommand *command, char * const *args);
-int show_config_action            (NihCommand *command, char * const *args);
-int check_config_action           (NihCommand *command, char * const *args);
-int usage_action                  (NihCommand *command, char * const *args);
-int notify_disk_writeable_action  (NihCommand *command, char * const *args);
-int get_env_action                (NihCommand *command, char * const *args);
-int set_env_action                (NihCommand *command, char * const *args);
-int list_env_action               (NihCommand *command, char * const *args);
-int unset_env_action              (NihCommand *command, char * const *args);
-int reset_env_action              (NihCommand *command, char * const *args);
-int list_sessions_action          (NihCommand *command, char * const *args);
+int start_action                         (NihCommand *command, char * const *args);
+int stop_action                          (NihCommand *command, char * const *args);
+int restart_action                       (NihCommand *command, char * const *args);
+int reload_action                        (NihCommand *command, char * const *args);
+int status_action                        (NihCommand *command, char * const *args);
+int list_action                          (NihCommand *command, char * const *args);
+int emit_action                          (NihCommand *command, char * const *args);
+int reload_configuration_action          (NihCommand *command, char * const *args);
+int version_action                       (NihCommand *command, char * const *args);
+int log_priority_action                  (NihCommand *command, char * const *args);
+int show_config_action                   (NihCommand *command, char * const *args);
+int check_config_action                  (NihCommand *command, char * const *args);
+int usage_action                         (NihCommand *command, char * const *args);
+int notify_disk_writeable_action         (NihCommand *command, char * const *args);
+int notify_dbus_address_action           (NihCommand *command, char * const *args);
+int notify_cgroup_manager_address_action (NihCommand *command, char * const *args);
+int get_env_action                       (NihCommand *command, char * const *args);
+int set_env_action                       (NihCommand *command, char * const *args);
+int list_env_action                      (NihCommand *command, char * const *args);
+int unset_env_action                     (NihCommand *command, char * const *args);
+int reset_env_action                     (NihCommand *command, char * const *args);
+int list_sessions_action                 (NihCommand *command, char * const *args);
 
 /**
  * use_dbus:
@@ -333,7 +335,7 @@ upstart_open (const void *parent)
 
 	user_addr = getenv ("UPSTART_SESSION");
 
-	if (user_addr && dbus_bus_type < 0) {
+	if (user_addr && user_addr[0] && dbus_bus_type < 0) {
 		user_mode = TRUE;
 	}
 
@@ -1033,9 +1035,33 @@ reload_action (NihCommand *  command,
 
 	job->auto_start = FALSE;
 
-	if (job_reload_sync (NULL, job) < 0)
-	        goto error;
+	if (job_reload_sync (NULL, job) < 0) {
+		/* well reload_sync call should always work...  unless we
+		 * didn't reboot since upgrade and pid1 is still old
+		 * upstart that does not have reload_sync call, fallback
+		 * to sending SIGHUP to main process
+		 */
+		err = nih_error_get ();
+		if (strcmp(((NihDBusError *)err)->name, DBUS_ERROR_UNKNOWN_METHOD)) {
+			nih_error ("%s", err->message);
+			nih_free (err);
+			return 1;
+		}
+		nih_free (err);
 
+		if (job_get_processes_sync (NULL, job, &processes) < 0)
+			goto error;
+
+		if ((! processes[0]) || strcmp (processes[0]->item0, "main")) {
+			nih_error (_("Not running"));
+			return 1;
+		}
+
+		if (kill (processes[0]->item1, SIGHUP) < 0) {
+			nih_error_raise_system ();
+			goto error;
+		}
+	}
 	return 0;
 
 error:
@@ -1427,7 +1453,6 @@ set_env_action (NihCommand *command, char * const *args)
 {
 	nih_local NihDBusProxy  *upstart = NULL;
 	NihError                *err;
-	char                    *envvar;
 	nih_local char         **job_details = NULL;
 	int                      ret;
 
@@ -1444,14 +1469,12 @@ set_env_action (NihCommand *command, char * const *args)
 	if (! job_details)
 		return 1;
 
-	envvar = args[0];
-
 	upstart = upstart_open (NULL);
 	if (! upstart)
 		return 1;
 
-	ret = upstart_set_env_sync (NULL, upstart, job_details,
-			envvar, ! retain_var);
+	ret = upstart_set_env_list_sync (NULL, upstart, job_details,
+			args, ! retain_var);
 
 	if (ret < 0)
 		goto error;
@@ -1480,15 +1503,12 @@ unset_env_action (NihCommand *command, char * const *args)
 {
 	nih_local NihDBusProxy  *upstart = NULL;
 	NihError                *err;
-	char                    *name;
 	nih_local char         **job_details = NULL;
 
 	nih_assert (command != NULL);
 	nih_assert (args != NULL);
 
-	name = args[0];
-
-	if (! name) {
+	if (! args[0]) {
 		fprintf (stderr, _("%s: missing variable name\n"), program_name);
 		nih_main_suggest_help ();
 		return 1;
@@ -1502,7 +1522,7 @@ unset_env_action (NihCommand *command, char * const *args)
 	if (! upstart)
 		return 1;
 
-	if (upstart_unset_env_sync (NULL, upstart, job_details, name) < 0)
+	if (upstart_unset_env_list_sync (NULL, upstart, job_details, args) < 0)
 		goto error;
 
 	return 0;
@@ -2030,6 +2050,51 @@ error:
 	return 1;
 }
 
+/**
+ * notify_cgroup_manager_address_action:
+ * @command: NihCommand invoked,
+ * @args: command-line arguments.
+ *
+ * This function is called for the "notify-cgroup-manager-address" command.
+ *
+ * Returns: command exit status.
+ **/
+int
+notify_cgroup_manager_address_action (NihCommand    *command,
+				      char * const  *args)
+{
+	nih_local NihDBusProxy   *upstart = NULL;
+	NihError                 *err;
+	char                     *address = NULL;
+
+	nih_assert (command != NULL);
+	nih_assert (args != NULL);
+
+	if (! args[0]) {
+		fprintf (stderr, _("%s: missing address\n"), program_name);
+		nih_main_suggest_help ();
+		return 1;
+	}
+
+	address = args[0];
+
+	upstart = upstart_open (NULL);
+	if (! upstart)
+		return 1;
+
+	if (upstart_notify_cgroup_manager_address_sync (NULL, upstart, address) < 0)
+		goto error;
+
+	return 0;
+
+error:
+	err = nih_error_get ();
+	nih_error ("%s", err->message);
+	nih_free (err);
+
+	return 1;
+}
+
 /**
  * list_sessions_action:
  * @command: NihCommand invoked,
@@ -3223,13 +3288,13 @@ static NihCommand commands[] = {
 	  &env_group, reset_env_options, reset_env_action },
 
 	{ "set-env", N_("VARIABLE[=VALUE]"),
-	  N_("Set a job environment variable."),
-	  N_("Adds or updates a variable in the job environment table."),
+	  N_("Set one or more job environment variables."),
+	  N_("Adds or updates variables in the job environment table."),
 	  &env_group, set_env_options, set_env_action },
 
 	{ "unset-env", N_("VARIABLE"),
-	  N_("Remove a job environment variable."),
-	  N_("Discards the specified variable from the job environment table."),
+	  N_("Remove one or more job environment variables."),
+	  N_("Discards variables from the job environment table."),
 	  &env_group, unset_env_options, unset_env_action },
 
 	{ "usage",  N_("JOB"),
@@ -3237,10 +3302,15 @@ static NihCommand commands[] = {
 	  N_("JOB is the name of the job which usage is to be shown.\n" ),
 	  NULL, usage_options, usage_action },
 
+	{ "notify-cgroup-manager-address", NULL,
+	  N_("Inform Upstart of D-Bus address cgroup manager is available on."),
+	  N_("Run to allow Upstart to provide cgroup stanza support."),
+	  NULL, NULL, notify_cgroup_manager_address_action },
+
 	{ "notify-dbus-address", NULL,
 	  N_("Inform Upstart of D-Bus address to connect to."),
 	  N_("Run to allow Upstart to provide services over D-Bus."),
-	  NULL, NULL, notify_dbus_address_action},
+	  NULL, NULL, notify_dbus_address_action },
 
 	{ "notify-disk-writeable", NULL,
 	  N_("Inform Upstart that disk is now writeable."),
