@@ -366,7 +366,9 @@ main (int   argc,
 
 
 	/* Check we're root, or setuid root */
-	setuid (geteuid ());
+	if (setuid (geteuid ()) < 0)
+	    nih_warn (_("Couldn't set uid."));
+
 	if (getuid ()) {
 		nih_fatal (_("Need to be root"));
 		exit (1);
